/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

#include <ScatterWeb.System.h>
#include <ScatterWeb.CC1020.h>
#include <ScatterWeb.Accelerator.h>
#include <ScatterWeb.SensorBoard.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

extern void showInfos();
extern void SetDefMouseEncValues();
extern void GetAcceleratorValue(void* c);
extern void Calibration(uint16_t interval, uint16_t runOnce);
extern void StartWatch(int16_t sensitySoft_x,
			    int16_t sensitySoft_y,
			    int16_t sensitySoft_z,
			    uint16_t loopInterval,
			    uint16_t mouseStepRange1,
			    uint16_t mouseStepRange2);









void CallMe();


bool Process_radioHandler(struct netpacket_handler_args* args)
{
	/*
		Notes: * The received packet data becomes invalid when this handler is completed.
	*/

	// Pass packet through command interface
	if ( Cmd_radioHandler( args ) ) {
		// packet handled by messaging module
		return true;
	}

	if ( args->cpaResult == ADRCOMP_OTHER ) {
		// intercept packets for other nodes here
	} else if ( args->cpaResult == ADRCOMP_BROADCAST ) {
		// process broadcast packets here
	} else {
		// process packets for this node here
		switch ( args->netheader->type ) {
			case PING_REQUEST_PACKET:
				// send back to receiver's address
				Net_sendPong(args->netheader->from);
				break;

			case PING_REPLY_PACKET:
				LOG_LOW( "PONG from %u", args->netheader->from );
				break;

			default:
				return false;		// packet not handled (unknown)
		}
		return true;				// packet handled
	}
	return false;					// packet not handled (other destination)
}


void Process_sensorHandler( void* sdata )
{
	
	sdata_t *s = (sdata_t*) sdata;
	
	if (s->channel == 10){
		button1=0x10;
		sendMData();
	}
	if (s->channel == 11){
		button2=0x01;
		sendMData();
	}
	
}

void Process_init()
{
	System_registerCallback( C_RADIO, ( fp_vp )Process_radioHandler );
	System_registerCallback( C_SENSOR, ( fp_vp )Process_sensorHandler );
	
	//Das hier wird als Init ausgefhrt
	SetDefMouseEncValues(); //Standardwerte fr mouse-Variable eintragen 
	printf("//Bereit... \r\n");
	
	button1=0x00;
	button2=0x00;
	
	Data_init(SensorBoard_init);
	Data_enable(5, 0, NULL);
	Data_enable(10,0,NULL);
	Data_enable(11,0,NULL);
	
	//StartWatch(50);
	//Timers_add(5*1024, CallMe,  0xFFFF); //eine Sekunde ergibt 1024 Ticks
}



/**
 * @brief	Ping request
 *
 * Send a ping request to another node
 * Example: ping 24
 */
COMMAND( ping, 0, cmdargs )
{
	uint16_t node;

	node = String_parseUint16( cmdargs->args, NULL );
	if ( node > 0 ) {
		Net_sendPing((netaddr_t)node);
	}
}

/**
 * @brief Echo Arguments
 *
 * Testcase 1: long command name
 * Testcase 2: argument splitting
 */
COMMAND( echoargs, CMDFLAG_SERIAL, cmdargs )
{
	uint16_t  i;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	char* arg = cmdargs->args;

	for ( i = 1; i <= count; i++ ) {
		printf("%u: %*s\n", i, strlen(arg), arg);
		arg = ( char* )Cmd_GetNextArg( arg );
	}
	snprintf(cmdargs->response_args, cmdargs->response_bufsize, "%u args found", count );
}



//-------------------------------------------------------------------------------------------------------------------
//Eigener Code
//-------------------------------------------------------------------------------------------------------------------






COMMAND( Hi, CMDFLAG_SERIAL, cmdargs){
	printf( "Hi Chris es funktioniert!\r\n");
	uint16_t n = String_parseUint16(cmdargs->args, NULL);
	uint16_t i = 0;
	for (i=1;i<=n;i++){
		printf("Das ist Zeile %u \r\n", i);
	}
}

COMMAND(GetAVal, CMDFLAG_SERIAL, cmdargs){
	uint16_t n = String_parseUint16(cmdargs->args, NULL);
	GetAcceleratorValue(n);
}


/*Erwartete Parameter: <int sensitysoft_x>
					   <int sensitysoft_y>
					   <int sensitysoft_z>
					   <int LoopInterval> 
					   <int mouseStepRange1>
					   <int mouseStepRange2>
					   <int mouseStepRange3>      */
COMMAND(StartWatch, CMDFLAG_SERIAL, cmdargs){
	char* args = cmdargs->args;
	char* p1;
	char* p2;
	char* p3;
	char* p4;
	char* p5;
	char* p6;
	
	Cmd_SplitArgs(args);
	p1 = args;

	args = Cmd_GetNextArg(args);
	p2 = args;
	
	args = Cmd_GetNextArg(args);
	p3 = args;
	
	args = Cmd_GetNextArg(args);
	p4 = args;
	
	args = Cmd_GetNextArg(args);
	p5 = args;
		
	args = Cmd_GetNextArg(args);
	p6 = args;
			
	
	int16_t n1 = strtol(p1, NULL, 10);
	int16_t n2 = strtol(p2, NULL, 10);
	int16_t n3 = strtol(p3, NULL, 10);
	uint16_t n4 = String_parseUint16(p4, NULL);
	uint16_t n5 = String_parseUint16(p5, NULL);
	uint16_t n6 = String_parseUint16(p6, NULL);
	
	/*if (){
		printf("//Erwartete Parameter: StartWatch <int sensitySoft_x .. _z> <int LoopInterval>  \r\n");
		return;
	}*/
	
	StartWatch(n1, n2, n3, n4, n5, n6);
}

COMMAND(StopWatch, CMDFLAG_SERIAL, cmdargs){
	//uint16_t n = String_parseUint16(cmdargs->args, NULL);
	stopWatch=True;
}

/* Fhrt eine Kalibrierung einmal oder unendlich oft aus
 * Parameter: <RunOnce> <Interval>*/
COMMAND(Calibrate, CMDFLAG_SERIAL, cmdargs){
	//Parameter auslesen
	char* args = cmdargs->args;
	char* p1;
	char* p2;
	
	Cmd_SplitArgs(args);
	p1 = args;

	args = Cmd_GetNextArg(args);
	p2 = args;
	
	uint16_t n1 = String_parseUint16(p1, NULL);
	uint16_t n2 = String_parseUint16(p2, NULL);
	
	//Funktion aufrufen
	Calibration(n1, n2);
	
}

//Stoppt die Calibration Loop
COMMAND(StopCalibration, CMDFLAG_SERIAL, cmdargs){
	stopCal=True;
}


COMMAND(ShowInfos, CMDFLAG_SERIAL, cmdargs){
	showInfos();
}

COMMAND(LEDON, CMDFLAG_SERIAL, cmdargs){
	LED_ON();
}

COMMAND(LEDOFF, CMDFLAG_SERIAL, cmdargs){
	LED_OFF();
}


// Software header.
APP(
	APPH_STATIC_APP,	// Application Type
	__APP_NAME,			// Application Name (8 chars)
	1, 0				// Aplication Version (2 bytes)
);
