/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

#include <string.h>
#include <ScatterWeb.System.h>
#include <ScatterWeb.CC1020.h>
#include "ScatterWeb.Uart.h"

uint16_t values[3];
uint32_t counter = 0;

void print_adc() {
	uint8_t		hexval[4];
	int i;
	
	String_writeHex(putchar, &counter, 4);
	String_writeHex(putchar, values, 6);		

	UART_WAIT_TX();
	UART_TX = '\n';
	
	/*
		Example of parsing the output in C#:
	
		string inputString = "A5000000F407E8078E0B";
		byte[] inputRaw = new byte[inputString.Length / 2];
		for( int i = 0; i < inputRaw.Length; i++ )
			inputRaw[i] = Convert.ToByte(inputString.Substring(i*2,2), 16);
		UInt32 counter = BitConverter.ToUInt32( inputRaw, 0 );
		UInt16[] values = new UInt16[3];
		values[0] = BitConverter.ToUInt16( inputRaw, 4 );
		values[1] = BitConverter.ToUInt16( inputRaw, 6 );
		values[2] = BitConverter.ToUInt16( inputRaw, 8 );	
	*/
}

interrupt(NOVECTOR) adcirq(void) {
	P4OUT |= 0x02;

	values[0] = ADC12MEM3;
	
	values[0] = ADC12MEM0;
	values[1] = ADC12MEM1;
	values[2] = ADC12MEM2;
	counter++;
	
	print_adc();
	P4OUT &= ~0x02;
	
	//_BIC_SR_IRQ(LPM0_bits);               // Clear LPM0
}

void configure_adc(void) {
	_DINT();
	ADC12IFG = 0;
	ADC12IE = 0;
	ADC12CTL0 = 0;				// disable ADC
	
	P6SEL |= 0x07;				// select adc function
	
	P4SEL &= ~0x02;				// configure pin 4.1 to test sampling rate
	P4DIR |= 0x02;
	
	ADC12MCTL0 = ( INCH_0 + SREF_0 );           // P60: AIN0
	ADC12MCTL1 = ( INCH_1 + SREF_0 );           // P61: AIN0
	ADC12MCTL2 = ( INCH_2 + SREF_0 );		    // P62: AIN0
	
	/* 4th channel is used as delay to adjust to 1.0 kHz */
	ADC12MCTL3 = ( INCH_3 + SREF_0 ) + EOS;     // P62: AIN0

	ADC12CTL0 = ADC12ON
				+ MSC
				+ SHT0_6;		// sample 128 ticks
	ADC12CTL1 = SHP
				| ADC12DIV_7	// divisor 8
				| CSTARTADD_0	// start at register 0
				| ADC12SSEL_2	// clock source MCLOCK
				| CONSEQ_3;		// single sequence of channels

	/*
		calculated: (4505600 Hz / 8) / (4 * 128) = 1100,00 Hz
		measured: 1.0 kHz
	*/

	SET_ISR( ADC_VECTOR, adcirq );
	ADC12IE = 8;			// enable interrupt on channel 2
	ADC12CTL0 |= ENC;     	// enable conversion
	_EINT();
}

void Process_init()
{
	WDTCTL = WDTPW+WDTHOLD;
	
	uart_set_speed(UART_MODE_RS232, 230400);
		
	printf("\r\nUART 230400 bps\r\n");
	
	configure_adc();
	ADC12CTL0 |= ADC12SC;		// Start conversion
	while(1) {
		//_BIS_SR(LPM0_bits);         // Enter LPM0
		//print_adc();
		//System_wait(10000);			// REMOVE!
	}
}


// Software header.
APP(
    APPH_STATIC_APP,	// Application Type
    __APP_NAME,			// Application Name (8 chars)
    1, 0				// Aplication Version (2 bytes)
);
